﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using System.Web.Mvc;
using TpmDAO.Entity;
using TpmDAO.Persistence;
using TpmMVC.Areas.RestrictedArea.Models.Facility;
using TpmMVC.Controllers;
using TpmMVC.Util;

namespace TpmMVC.Areas.RestrictedArea.Controllers
{
    [Authorize(Order = 0, Roles = "Admin, Premium")]
    public class FacilityController : BaseController
    {
        private int UserId { get; set; } = 0;

        #region Views

        // GET: RestrictedArea/Facility
        public ActionResult Index()
        {
            FillDropDown();
            return View();
        }

        [HttpGet]
        public ActionResult Create()
        {
            try
            {
                ViewBag.ShowFooter = true;
                ViewBag.ShowPictures = true;
                ViewBag.ShowAddress = false;
                ViewBag.Title = "Nova Instalação";
                FillDropDown();
                return PartialView();
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao obter Proprietários - " + e.Message;
                return RedirectToAction("Index");
            }
        }

        #endregion

        #region Save

        [HttpPost]
        public async Task<ActionResult> SaveFacilityAsync(FacilityViewModel Model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var Facility = await FillFacilityObject(Model);
                    FacilityDAO.Insert(Facility);
                    return GetSuccess("Instalação salva com sucesso!");
                }
                else
                {
                    ViewBag.ShowFooter = true;
                    ViewBag.ShowPictures = true;
                    ViewBag.ShowAddress = true;
                    ViewBag.Title = "Nova Instalação";
                    FillDropDown(Convert.ToInt32(Model.State));
                    return PartialView("Create", Model);
                }
            }
            catch (Exception e)
            {
                return GetError("Erro ao salvar instalação - " + e.Message);
            }
        }

        #endregion

        #region Update

        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<ActionResult> UpdateFacilityAsync(FacilityViewModel Model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var Facility = await FillFacilityObject(Model);
                    AddressDAO.Update(Facility.Address);
                    var Pictures = Facility.Pictures;
                    Facility.Pictures = null;
                    FacilityDAO.Update(Facility);

                    foreach (var item in Pictures)
                        FacilityPictureDAO.Insert(item);

                    return GetSuccess("Instalação salva com sucesso!");
                }
                else
                {
                    FillDropDownsUser();
                    FillDropDown();

                    var ErrorMessages = "";
                    var Errors = ModelState.Values.SelectMany(v => v.Errors.Select(b => b.ErrorMessage)).ToList();

                    Errors.ForEach(a => ErrorMessages += a + "<br />");

                    return GetError(ErrorMessages);
                }
            }
            catch (Exception e)
            {
                return GetError("Erro ao alterar dados de instalação! - " + e.Message);
            }
        }

        #endregion

        #region List

        [HttpGet]
        public JsonResult ListFacilities()
        {
            try
            {
                var Facilities = FillGridFacility();
                return Json(Facilities, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao listar intalações - " + e.Message);
            }
        }

        #endregion

        #region Active/Deactive

        [HttpGet]
        public ActionResult RemoveFacility(int IdFacility)
        {
            try
            {
                var Facility = FacilityDAO.FindByKey(IdFacility);
                Facility.Status = false;
                FacilityDAO.Update(Facility);
                TempData["Success"] = "Instalação desativada com sucesso!";
                return RedirectToAction("Index");
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao desativar instalação! - " + e.Message;
                return RedirectToAction("Index");
            }
        }


        [HttpGet]
        public ActionResult ReactiveFacility(int IdFacility)
        {
            try
            {
                var Facility = FacilityDAO.FindByKey(IdFacility);
                Facility.Status = true;
                FacilityDAO.Update(Facility);
                TempData["Success"] = "Instalação reativada com sucesso!";
                return RedirectToAction("Index");
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao reativar instalação! - " + e.Message;
                return RedirectToAction("Index");
            }
        }

        #endregion

        #region GetInfo

        public ActionResult GetFacilityInfo(int Param)
        {
            try
            {
                ViewBag.ShowFooter = false;
                ViewBag.ShowPictures = false;
                ViewBag.ShowAddress = true;
                ViewBag.Title = "Informações da instalação";

                var Model = FillFacilityModel(Param);

                FillDropDown(Convert.ToInt32(Model.State));

                return PartialView("Create", Model);
            }
            catch (Exception e)
            {
                return GetError("Erro ao obter informações sobre a instalação! - " + e.Message);
            }
        }

        public JsonResult GetFacilityData(int Param)
        {
            try
            {
                var Model = FillFacilityModel(Param);
                return Json(Model, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao obter informações sobre a instalação! - " + e.Message);
            }
        }

        #endregion

        #region Fill

        private List<FacilityGridViewModel> FillGridFacility()
        {
            try
            {
                if (User.IsInRole("Premium"))
                    UserId = UserData.Person.IdPerson;

                var Facilities = FacilityDAO.FindAllComplete(UserId);
                return Facilities.Select(x => new FacilityGridViewModel
                {
                    IdFacility = x.IdFacility,
                    Address = x.Address.ToString(),
                    Name = x.NameFacility,
                    Owner = PersonDAO.FindByKey(x.Owner).FullName,
                    Status = x.Status ? "Ativo" : "Inativo",
                    Active = x.Status ? true : false,
                }).ToList();
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private async Task<Facility> FillFacilityObject(FacilityViewModel Model)
        {
            var Pictures = new List<FacilityPicture>();
            try
            {
                var Facility = new Facility
                {
                    NameFacility = Model.Name,
                    Description = Model.Descript,
                    Email = Model.Email,
                    IdAddress = Model.IdAddress ?? 0,
                    IdFacility = Convert.ToInt32(Model.IdFacility),
                    Owner = Convert.ToInt32(Model.IdOwner),
                    Phone = RemoveCharsNonNumeric(Model.Phone),
                    Status = Model.IdFacility.HasValue && Model.IdFacility != 0 ? Model.Status.Value : true,
                    Address = new Address
                    {
                        IdAddress = Model.IdAddress ?? 0,
                        Detail = Model.Detail,
                        IdCity = Convert.ToInt32(Model.City),
                        Neighborhood = Model.Neighborhood,
                        Number = Model.Number,
                        Street = Model.Street,
                        ZipCode = RemoveCharsNonNumeric(Model.Cep),
                    },
                    Pictures = new List<FacilityPicture>()
                };
                if (Model.FacilityPrictures[0] != null)
                {
                    foreach (var item in Model.FacilityPrictures)
                    {
                        var Picture = new FacilityPicture
                        {
                            IdFacility = Convert.ToInt32(Model.IdFacility),
                            Path = await Images.UploadPicture(item, Helpers.GenerateFileName(item.FileName), "facility")
                        };
                        Pictures.Add(Picture);
                    }
                }
                Facility.Pictures = Pictures;
                var GeoLocation = await Helpers.GetGeoPositionAsync(Facility.Address);
                if (GeoLocation != null)
                {
                    Facility.Address.Latitude = GeoLocation.Lat;
                    Facility.Address.Longitude = GeoLocation.Lng;
                }

                return Facility;
            }
            catch (Exception e)
            {
                foreach (var item in Pictures)
                    await Util.Images.DeletePicture(item.Path, "facility");
                throw new Exception(e.Message);
            }
        }

        private FacilityViewModel FillFacilityModel(int IdFacility)
        {
            try
            {
                var Facility = FacilityDAO.FindByKeyComplete(IdFacility);
                var FacilityVM = new FacilityViewModel
                {
                    Cep = Facility.Address.ZipCode,
                    City = Facility.Address.IdCity.ToString(),
                    IdAddress = Facility.IdAddress,
                    Neighborhood = Facility.Address.Neighborhood,
                    Number = Facility.Address.Number,
                    State = Facility.Address.City.IdState.ToString(),
                    Street = Facility.Address.Street,
                    Name = Facility.NameFacility,
                    Email = Facility.Email,
                    Detail = Facility.Address.Detail,
                    Phone = Facility.Phone,
                    Status = Facility.Status,
                    IdFacility = Facility.IdFacility,
                    IdOwner = Facility.Owner.ToString(),
                    Descript = Facility.Description,
                    Urls = Facility.Pictures?.Select(x => x.Path).ToList()
            };

                return FacilityVM;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region Fill DropDown

        private void FillDropDown(int IdState = 0)
        {
            try
            {
                if (User.IsInRole("Premium"))
                    UserId = UserData.Person.IdPerson;

                ViewData["Owner"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryUsers(UserId));
                ViewData["States"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryStates());
                if (IdState != 0)
                    ViewData["Cities"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryCities(IdState));
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion
    }
}