﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using TpmDAO.Entity;
using TpmDAO.Persistence;
using TpmMVC.Areas.RestrictedArea.Models.User;
using TpmMVC.Models;
using TpmMVC.Controllers;

namespace TpmMVC.Areas.RestrictedArea.Controllers
{
    [Authorize(Roles = "Admin")]
    public class UserController : BaseController
    {
        private int UserId { get; set; } = 0;

        #region Views
        // GET: RestrictedArea/User
        public ActionResult Users()
        {
            FillDropDownsUser();
            return View();
        }

        #endregion

        #region Update

        [HttpPost]
        public ActionResult UpdateUser(UserViewModel Model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var User = FillUserModel(Model);
                    AddressDAO.Update(User.Person.Address);
                    PersonDAO.Update(User.Person);
                    UserDAO.Update(User);

                    foreach (var item in User.UserRoles)
                    {
                        if (item.IsNew)
                        {
                            item.IsNew = false;
                            item.User = null;
                            UserRoleDAO.Insert(item);
                        }
                        else
                            UserRoleDAO.Update(item);
                    }

                    TempData["Success"] = "Usuário atualizado com sucesso!";
                    return RedirectToAction("Users");
                }
                else
                {
                    FillDropDownsUser(Model.State);
                    return View("Users", Model);
                }
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao atualizar informações do usuário - " + e.Message;
                return RedirectToAction("Users");
            }
        }

        #endregion

        #region Deactive/Reactive User

        [HttpGet]
        public ActionResult RemoveUser(int IdUser)
        {
            try
            {
                var User = UserDAO.FindByKeyComplete(IdUser);
                var RolePremium = User.UserRoles.Where(x => x.IdRole == 2 && x.Status).FirstOrDefault();
                var Role = User.UserRoles.Where(x => x.IdRole == 3).FirstOrDefault();

                if (RolePremium != null)
                {
                    RolePremium.Status = false;
                    UserRoleDAO.Delete(RolePremium);
                }
                if (Role != null)
                {
                    Role.Status = false;
                    UserRoleDAO.Update(Role);
                }
                TempData["Success"] = "O acesso do usuário: " + User.Person.FullName + " foi desativado com sucesso";

                return RedirectToAction("Users");
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao desativar acesso - " + e.Message;
                return RedirectToAction("Users");
            }
        }

        [HttpGet]
        public ActionResult ReactiveUser(int IdUser)
        {
            try
            {
                var User = UserDAO.FindByKeyComplete(IdUser);
                var Role = User.UserRoles.Where(x => x.IdRole == 3 && !x.Status).FirstOrDefault();
                Role.Status = true;
                UserRoleDAO.Update(Role);
                TempData["Success"] = "O acesso do usuário: " + User.Person.FullName + " foi reativado com sucesso";
                return RedirectToAction("Users");
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao reativar usuário - " + e.Message;
                return RedirectToAction("Users");
            }
        }

        #endregion

        #region Json

        /// <summary>
        /// Get a list of users with all their information
        /// </summary>
        /// <returns>List of user objects</returns>
        [HttpGet]
        public JsonResult ListUsers()
        {
            try
            {
                var Return = FillGridUsers();
                return Json(Return, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao listar usuários - " + e.Message);
            }
        }

        /// <summary>
        /// Get info for an user
        /// </summary>
        /// <param name="Param">User ID</param>
        /// <returns>Json User object </returns>
        [HttpGet]
        public JsonResult GetUserInfo(string Param)
        {
            try
            {
                var User = FillViewUserInfo(Param);
                return Json(User, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao obter dados do usuário! - " + e.Message);
            }
        }

        #endregion

        #region Fill

        /// <summary>
        /// Fill User's grid view model
        /// </summary>
        /// <returns>List of User grid view model</returns>
        private List<UserGridViewModel> FillGridUsers()
        {
            try
            {
                var Users = UserDAO.FindAllComplete();
                var ListUsers = Users.Select(x => new UserGridViewModel
                {
                    IdUser = x.IdUser,
                    Name = x.Person.FullName,
                    Email = x.Email,
                    RegisterDate = x.UserRoles.Where(a => a.IdRole == 3).Select(b => b.RequisitionDate).FirstOrDefault().ToShortDateString(),
                    Type = x.UserRoles.Where(a => a.IdRole == 2 && a.Status).FirstOrDefault() != null ? "Premium" : "Usuário",
                    Status = x.UserRoles.Where(a => a.Status).ToList().Count > 0 ? "Ativo" : "Desativado",
                    Active = x.UserRoles.Where(a => a.Status).ToList().Count > 0 ? true : false
                }).ToList();

                return ListUsers.OrderBy(q => q.Name).ToList();
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private User FillUserModel(UserViewModel Model)
        {
            try
            {

                var User = UserDAO.FindByKeyComplete(Model.IdUser);
                User.Email = Model.Email;

                #region Person

                User.Person = new Person
                {
                    BirthDate = DateTime.Parse(Model.BirthDateFormat),
                    Class = CategoryClassDAO.FindByKey(Convert.ToInt32(Model.IdClass)).Class,
                    FirstName = Model.Name,
                    LastName = Model.LastName,
                    Phone = RemoveCharsNonNumeric(Model.Tel),
                    Gender = Model.Gender,
                    IdHand = Convert.ToInt32(Model.IdHand),
                    IdPerson = User.IdPerson,
                    IdAddress = User.Person.IdAddress,
                    Address = new Address
                    {
                        IdAddress = User.Person.IdAddress.Value,
                        IdCity = Model.City,
                        Street = Model.Street,
                        Detail = Model.Detail,
                        Neighborhood = Model.Neighborhood,
                        Number = Model.Number,
                        ZipCode = RemoveCharsNonNumeric(Model.Cep),
                    }
                };
                #endregion

                #region Roles

                if (Model.PremiumAccess)
                {
                    if (User.UserRoles.Where(x => x.IdRole == 2).FirstOrDefault() != null)
                        User.UserRoles.Where(x => x.IdRole == 2).FirstOrDefault().Status = true;
                    else
                        User.UserRoles.Add(
                            new UserRole
                            {
                                IdRole = 2,
                                IdUser = User.IdUser,
                                IsNew = true,
                                Status = true,
                            }
                            );
                }
                else
                {
                    if (User.UserRoles.Where(x => x.IdRole == 2).FirstOrDefault() != null)
                        User.UserRoles.Where(x => x.IdRole == 2).FirstOrDefault().Status = false;
                }

                if (Model.UserAccess)
                {
                    if (User.UserRoles.Where(x => x.IdRole == 3).FirstOrDefault() != null)
                        User.UserRoles.Where(x => x.IdRole == 3).FirstOrDefault().Status = true;
                }
                else
                {
                    if (User.UserRoles.Where(x => x.IdRole == 3).FirstOrDefault() != null)
                        User.UserRoles.Where(x => x.IdRole == 3).FirstOrDefault().Status = false;
                }

                #endregion

                return User;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion
    }
}