﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Web.Mvc;
using TpmDAO.Entity;
using TpmDAO.Persistence;
using TpmMVC.Areas.RestrictedArea.Models.User;
using TpmMVC.Models;
using TpmMVC.Util;
using TpmMVC.Util.CustomValidators.Session;
using Utils;

namespace TpmMVC.Controllers
{
    [SessionExpire]
    public class BaseController : Controller
    {
        protected User UserData { get; } = SessionManager.GetUserData();

        public BaseController()
        {
            if (System.Web.HttpContext.Current.Request.IsAuthenticated)
            {
                ViewBag.UserName = UserData.Person.FirstName;
                ViewBag.Photo = UserData.ProfilePicture;
            }
        }

        #region Delete Image

        public async Task<JsonResult> DeleteImageAsync(string Param)
        {
            try
            {
                await Images.DeletePicture(GetFileFromUrl(Param), "facility");
                var Picture = FacilityPictureDAO.FindByUrl(Param);
                if (Picture != null)
                    FacilityPictureDAO.Delete(Picture);
                return GetSuccess("Imagem removida com sucesso!");
            }
            catch (Exception e)
            {
                return GetError("Erro ao excluir imagem! - " + e.Message);
            }
        }

        #endregion

        #region Protected

        protected static string RemoveCharsNonNumeric(string Str)
        {
            return Regex.Replace(Str, @"[^\d]", String.Empty);
        }

        protected void ClearTempDataCache()
        {
            TempData.Remove("Error");
            TempData.Remove("Success");
        }

        protected JsonResult GetJson(Object Data)
        {
            try
            {
                JsonSerializerSettings Serializer = new JsonSerializerSettings { ReferenceLoopHandling = ReferenceLoopHandling.Ignore };
                var result = JsonConvert.SerializeObject(Data, Formatting.Indented, Serializer);
                return Json(result, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao obter dados - " + e.Message);
            }
        }

        protected string GetFileFromUrl(string oUrl)
        {
            try
            {
                var Url = Uri.UnescapeDataString(oUrl);
                Regex regex = new Regex("facility/(.*?)\\?alt");
                return regex.Match(Url).Groups[1].ToString();
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        /// <summary>
        /// Get and fill User info to view model
        /// </summary>
        /// <param name="IdUser">User Id</param>
        /// <returns>UserViewModel object</returns>
        protected UserViewModel FillViewUserInfo(string UserId)
        {
            try
            {
                var User = new User();
                var isNumeric = int.TryParse(UserId, out var n);
                if (isNumeric)
                    User = UserDAO.FindByKeyComplete(Convert.ToInt32(UserId));
                else
                    User = UserDAO.FindByEmailComplete(UserId);
                if (User != null)
                {
                    var UserVM = new UserViewModel
                    {
                        IdUser = User.IdUser,
                        IdPerson = User.IdPerson,
                        BirthDate = User.Person.BirthDate.Value,
                        BirthDateFormat = User.Person.BirthDate.Value.ToString("yyy-MM-dd"),
                        IdClass = ((int)((Class)Enum.Parse(typeof(Class), User.Person.Class))).ToString(),
                        Email = User.Email,
                        Gender = User.Person.Gender,
                        IdHand = User.Person.IdHand.ToString(),
                        LastName = User.Person.LastName,
                        Name = User.Person.FirstName,
                        Tel = User.Person.Phone,
                        ProfilePicture = User.ProfilePicture,
                        UserAccess = User.UserRoles.Contains(User.UserRoles.Where(x => x.IdRole == 3 && x.Status).FirstOrDefault()),
                        PremiumAccess = User.UserRoles.Contains(User.UserRoles.Where(x => x.IdRole == 2 && x.Status).FirstOrDefault()),
                        Cep = User.Person.Address.ZipCode,
                        City = User.Person.Address.IdCity,
                        Detail = User.Person.Address.Detail,
                        Neighborhood = User.Person.Address.Neighborhood,
                        Number = User.Person.Address.Number,
                        State = User.Person.Address.City.IdState,
                        Street = User.Person.Address.Street,
                    };
                    return UserVM;
                }
                else
                    return null;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region GetMessage

        [AllowAnonymous]
        [HttpGet]
        public JsonResult GetSuccess(string Message, bool Redirect = false, string Url = "")
        {
            return Json(new FeedbackModel
            {
                Type = Models.Type.success,
                Message = Message,
                Redirect = Redirect,
                Url = Url
            }, JsonRequestBehavior.AllowGet);
        }

        [AllowAnonymous]
        [HttpGet]
        public JsonResult GetError(string Message, bool Redirect = false, string Url = "")
        {
            return Json(new FeedbackModel
            {
                Type = Models.Type.error,
                Message = Message,
                Redirect = Redirect,
                Url = Url
            }, JsonRequestBehavior.AllowGet);
        }

        [AllowAnonymous]
        [HttpGet]
        public JsonResult GetWarning(string Message, bool Redirect = false, string Url = "")
        {
            return Json(new FeedbackModel
            {
                Type = Models.Type.warning,
                Message = Message,
                Redirect = Redirect,
                Url = Url
            }, JsonRequestBehavior.AllowGet);
        }

        [AllowAnonymous]
        [HttpGet]
        public JsonResult GetInfo(string Message, bool Redirect = false, string Url = "")
        {
            return Json(new FeedbackModel
            {
                Type = Models.Type.info,
                Message = Message,
                Redirect = Redirect,
                Url = Url
            }, JsonRequestBehavior.AllowGet);
        }

        #endregion

        #region Json DropDowns

        [AllowAnonymous]
        [HttpGet]
        public JsonResult GetCitiesList(int IdState)
        {
            try
            {
                var Return = ComboEntitiesDAO.ReturnDictionaryCities(IdState);
                Return.Add("0", "Selecione");
                return Json(Return.OrderByDescending(x => x.Key.Equals("0")).ThenBy(x => x.Value), JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao buscar cidades - " + e.Message);
            }
        }

        #endregion

        #region Check Role

        [AllowAnonymous]
        protected ActionResult CheckRole(User User)
        {
            CustomRoleProvider RoleProvider = new CustomRoleProvider();
            try
            {
                var Roles = RoleProvider.GetAllRolesForUser(User.Email);
                if (Roles.Where(x => x.IdRole == 1 && x.Status && !x.IsNew).FirstOrDefault() != null)
                {
                    SessionManager.SetCookieToUser(User);
                    return RedirectToAction("Index", "Home", new { area = "RestrictedArea", id = 0 });
                }
                else if (Roles.Where(x => x.IdRole == 2 && x.Status && !x.IsNew).FirstOrDefault() != null)
                {
                    SessionManager.SetCookieToUser(User);
                    return RedirectToAction("Index", "Home", new { area = "RestrictedArea", id = 1 });
                }
                else if (Roles.Where(x => x.IdRole == 2 && !x.Status && x.IsNew).FirstOrDefault() != null)
                {
                    var Message = "Você já solicitou uma conta premium! Aguarde a aprovação!";
                    return RedirectToAction("Index", "UserLogin", new { msg = Message });
                }
                else if (Roles.Where(x => x.IdRole == 2 && !x.Status && !x.IsNew).FirstOrDefault() != null)
                {
                    var Message = "Seu acesso foi desativado! Solicite um novo acesso premium!";
                    return RedirectToAction("Index", "UserLogin", new { msg = Message });
                }
                else if (Roles.Count > 0 && Roles.Where(x => x.IdRole == 2).FirstOrDefault() == null)
                {
                    var Message = "Você ainda não possui conta Premium! Solicite abaixo!";
                    return RedirectToAction("Index", "UserLogin", new { msg = Message });
                }
                else
                {
                    var Message = "Você ainda não possui conta! Cadastra-se abaixo";
                    return RedirectToAction("Index", "UserLogin", new { msg = Message });
                }


            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao efetuar login - " + e.Message;
                return RedirectToAction("Index", "UserLogin");
            }
        }

        #endregion

        #region Error

        [AllowAnonymous]
        [HttpGet]
        public ActionResult Error(string msg)
        {
            try
            {
                ViewBag.Error = msg;

                return View("Error");
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region Fill DropDown

        /// <summary>
        /// Load all DropDownLists when manipulate user's views
        /// </summary>
        protected void FillDropDownsUser(int IdState = 0)
        {
            try
            {
                ViewData["States"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryStates());
                ViewData["Hands"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryHand());
                ViewData["Class"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryCategoryClass());
                if (IdState != 0)
                    ViewData["Cities"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryCities(IdState));


            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region Events

        [Authorize]
        [HttpGet]
        public JsonResult GetEvents()
        {
            try
            {
                var Events = FillEvents();

                return Json(Events, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private List<Event> FillEvents()
        {
            try
            {
                var IdOwner = 0;
                if (User.IsInRole("Premium"))
                    IdOwner = UserData.IdPerson;
                var Matches = MatchDAO.GetAllMatchesInPeriod(IdOwner);

                return Matches.Select(q => new Event
                {
                    date = q.DateMatch.HasValue ? q.DateMatch.Value.ToString("yyyy-MM-dd HH:mm:ss") : "",
                    description = CategoryDAO.GetDescript(q.Draw.IdCategory) + " <br/>" + (q.MatchStats.Count > 1 ? q.MatchStats.ElementAt(0).Person.FullName + " X " + q.MatchStats.ElementAt(1).Person.FullName :
                    q.MatchStats.Count == 1 ? q.MatchStats.FirstOrDefault().Person.FullName + " X " + "Indefinido" : "Indefinido X Indefinido" ),
                    title = q.Tournament.NameTournament + " - " + q.Round.NameRound,
                    type = "meeting",
                    url = "/RestrictedArea/Tournament/Index/" + q.Tournament.IdTournament

                }).ToList();
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

    }
}